public class LucaCompiler {

    public static org.apache.commons.cli.CommandLine parseOptions(String[] args) {
      org.apache.commons.cli.Options options = new  org.apache.commons.cli.Options();
      org.apache.commons.cli.CommandLineParser parser = new org.apache.commons.cli.PosixParser();
      options.addOption( "l", "lex", false,   "stop after lexical analysis and print results." );
      options.addOption( "p", "parse", false, "stop after syntax analysis and print results." );
      options.addOption( "s", "sem", false, "stop after semantic analysis and print results." );
      options.addOption( "t", "tree", false, "stop after tree code generation and print results." );
      options.addOption( "h", "help", false, "print help." );
      options.addOption( "v", "version", false, "print version." );
      options.addOption( "o", "output", true, "output file" );
      try {
        org.apache.commons.cli.CommandLine line = parser.parse( options, args );

        if (line.hasOption("h")) {
            org.apache.commons.cli.HelpFormatter formatter = new org.apache.commons.cli.HelpFormatter();
            formatter.printHelp( "lucac [options] -o file.vm file.luc", options );
            java.lang.System.exit(0);
        }

        if (line.hasOption("v")) {
            System.out.println("lucac, Version " + aux.Version.get() + ".");
            java.lang.System.exit(0);
        }

        return line;
     } catch( org.apache.commons.cli.ParseException exp ) {
       System.out.println( "Unexpected exception:" + exp.getMessage() );
     }

     return null;
    }

   public static void main (String args[]) throws Exception{
      org.apache.commons.cli.CommandLine options = parseOptions(args);

      String[] srcFiles = options.getArgs();
      if (srcFiles.length == 0) {
          System.err.println("Source file argument missing.");
          java.lang.System.exit(-1);
      } 
     
      if (options.hasOption("l")) {
	  lexer.Lex.main(srcFiles);
          java.lang.System.exit(-1);
      }

      if (options.hasOption("p")) {
	  parser.Parse.main(srcFiles);
          java.lang.System.exit(-1);
      }

      parser.Parse parser = new parser.Parse(srcFiles[0]);
      ast.PROGRAM e = parser.program();
      sem.Semantics.SemanticAnalysis(e);
      if (options.hasOption("s")) {
         java.lang.System.exit(0);
      }

      if (aux.Error.SemanticCount() == 0) {
         java.util.LinkedList treeCode = tree.Generate.GenTreeCode(e);
         if (options.hasOption("t")) {
            java.util.ListIterator j = treeCode.listIterator();
            while (j.hasNext()) {
               tree.Tree t = (tree.Tree) j.next();
               System.out.println(t.toString(0));
            }
            java.lang.System.exit(-1);
	 }
         String output = options.getOptionValue("output");
         if (output == null) {
	     System.err.println("Output file missing.");
             java.lang.System.exit(-1);
         }
         java.util.LinkedList vmCode = vm.Generate.GenVMCode(treeCode);
         // Write generated VM code to file here.
      }
   }   
}