package lexer;

class LexTable {
   public static int[][] NEXTSTATE = new int[100][];
   public static boolean[] ACCEPT = new boolean[100];
   public static int[] TOKEN = new int[100];
   public static boolean[][] ADVANCE = new boolean[100][];
   public static final int OTHER = -1;
   public static final int ANY = -2;
   public static final int ERRORSTATE = 99;
   public static final int STARTSTATE  = 0;

   static void add(int state, int token) {
      ACCEPT[state] = true;
      TOKEN[state] = token;
      ADVANCE[state] = new boolean[100];
   }

   static void add(int state, int[][] trans) {
      NEXTSTATE[state] = new int[100];
      ADVANCE[state] = new boolean[100];
      for(int i=0; i<trans.length; i++)
         if (trans[i][0] == OTHER) {
            for(int j=Char.FIRST; j<=Char.LAST; j++) {
               NEXTSTATE[state][j]=trans[i][1];
               ADVANCE[state][j] = false;
            }
         } else if (trans[i][0] == ANY) {
            for(int j=Char.FIRST; j<=Char.LAST; j++) {
               NEXTSTATE[state][j]=trans[i][1];
               ADVANCE[state][j] = true;
            }
         } else {
            NEXTSTATE[state][trans[i][0]]=trans[i][1];
            ADVANCE[state][trans[i][0]] = true;
         }
      ACCEPT[state] = false;
   }

   static {
      // state 0: strip blanks and tabs
      add(0, 
         new int[][]{
            {OTHER,1},
            {Char.ILLEGAL,ERRORSTATE},
            {Char.WHITESPACE,0},
            {Char.NEWLINE,0},
            {Char.EOF,36}
         });
      // state 1: operators
      add(1, 
         new int[][]{
         {Char.ILLEGAL,ERRORSTATE},
         {Char.PLUS,2},
         {Char.DASH,3},
         {Char.STAR,4},
         {Char.SLASH,5},
         {Char.PERCENT,6},
         {Char.COLON,7},
         {Char.BANG,8},
         {Char.COMMA,9},
         {Char.LBRACK,10},
         {Char.RBRACK,11},
         {Char.LPAREN,12},
         {Char.RPAREN,13},
         {Char.PERIOD,14},
         {Char.SEMICOLON,15},
         {Char.CARET,16},
         {Char.ATCHAR,17},
         {Char.BACKQUOTE,18},
         {Char.EQ,19},
         {Char.GT,20},
         {Char.LT,21},
         {Char.HASH,22},
         {Char.LETTER,29},
         {Char.Ee,29},
         {Char.DIGIT,31},
         {Char.DOUBLEQUOTE,33},
         {Char.SINGLEQUOTE,50},
      });
      add(2, Token.PLUS);
      add(3,  
         new int[][]{
            {OTHER,48},
            {Char.DASH,49}});
      add(4, Token.STAR);
      add(5, Token.SLASH);
      add(6, Token.PERCENT);
      add(7,  
         new int[][]{
            {OTHER,23},
            {Char.EQ,24}});
      add(8, Token.BANG);
      add(9, Token.COMMA);
      add(10, Token.LBRACK);
      add(11, Token.RBRACK);
      add(12,  
         new int[][]{
            {OTHER,44},
            {Char.STAR,45}});
      add(13, Token.RPAREN);
      add(14,  
         new int[][]{
            {OTHER,37},
            {Char.DIGIT,38}});
      add(15, Token.SEMICOLON);
      add(16, Token.CARET);
      add(17, Token.ATCHAR);
      add(18, Token.BACKQUOTE);
      add(19, Token.EQ);
      add(20,  
         new int[][]{
            {OTHER,25},
            {Char.EQ,26}});
      add(21,  
         new int[][]{
            {OTHER,27},
            {Char.EQ,28}});
      add(22, Token.NE);
      add(23, Token.COLON);
      add(24, Token.COLONEQ);
      add(25, Token.GT);
      add(26, Token.GE);
      add(27, Token.LT);
      add(28, Token.LE);
      // state 29: Identifier
      add(29, 
         new int[][]{
            {OTHER,30},
            {Char.LETTER,29},
            {Char.Ee,29},
            {Char.DIGIT,29}
         });
      add(30,Token.IDENT);
      // state 31: Numbers
      add(31, 
         new int[][]{
            {OTHER,32},
            {Char.DIGIT,31},
            {Char.PERIOD,38}
         });
      add(32,Token.INTLIT);
      // state 33: Strings
      add(33, 
         new int[][]{
            {ANY,33},
            {Char.DOUBLEQUOTE,34},
            {Char.NEWLINE,35},
            {Char.EOF,35}
         });
      add(34,Token.STRINGLIT);
      add(35,Token.ERROR_UNTERMINATED_STRING);
      // state 36: EOF
      add(36,Token.EOF);
      // state 37: PERIOD
      add(37, Token.PERIOD);
      // state 38: floating point starting with .
      add(38, 
         new int[][]{
            {OTHER,42},
            {Char.DIGIT,38},
            {Char.Ee,39}
         });
      add(39, 
         new int[][]{
            {ANY,43},
            {Char.PLUS,40},
            {Char.DASH,40},
            {Char.DIGIT,41},
         });
      add(40, 
         new int[][]{
            {ANY,43},
            {Char.DIGIT,41},
         });
      add(41, 
         new int[][]{
            {OTHER,42},
            {Char.DIGIT,41},
         });
      add(42, Token.REALLIT);
      add(43,Token.ERROR_REALLIT);
      // state 44: left parenthesis
      add(44, Token.LPAREN);
      // state 45: comments
      add(45, 
         new int[][]{
            {ANY,45},
            {Char.EOF,47},
            {Char.STAR,46},
         });
      add(46, 
         new int[][]{
            {ANY,45},
            {Char.EOF,47},
            {Char.STAR,46},
            {Char.RPAREN,0},
         });
      add(47, Token.ERROR_UNTERMINATED_COMMENT);
      // state 48: minus
      add(48, Token.MINUS);
      // state 49: line comments
      add(49, 
         new int[][]{
            {ANY,49},
            {Char.EOF,0},
            {Char.NEWLINE,0},
         });
      // state 50: Character literal
      add(50, 
         new int[][]{
            {ANY,51},
            {Char.SINGLEQUOTE,54},
            {Char.NEWLINE,53},
            {Char.EOF,53},
         });
      add(51, 
         new int[][]{
            {ANY,53},
            {Char.SINGLEQUOTE,52}
         });
      add(52,Token.CHARLIT);
      add(53,Token.ERROR_UNTERMINATED_CHAR);
      add(54,Token.ERROR_EMPTY_CHAR);
      // state 99: error
      add(99, Token.ERROR_ILLEGAL_CHARACTER);
   }

   static StringBuffer value = new StringBuffer();
   static int position;

   public static void action(int state, Input input) {
      int c = input.current();
      if (state == 0) 
         position = input.position();
      else if (state == 1) {
         value = new StringBuffer(100).append((char)c);
         position = input.position();
      } else if (ADVANCE[state][Char.classify(c)])
         value.append((char)c);
      else if (state == 34 || state == 52) 
         value = new StringBuffer(value.substring(1,value.length()-1));
   }

   static int tokenStart () {
      return position;
   }

   static String tokenValue() {
      return value.toString();
   }
}
